@@ -174,9 +174,15 @@ static int set_connected(struct connman_device *device,
 					CONNMAN_SERVICE_STATE_CONFIGURATION);
 		}
 	} else {
+		struct connman_service *service;
+
 		connman_element_unregister_children(&device->element);
 
 		device->disconnected = TRUE;
+
+		service = __connman_service_lookup_from_device(device);
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_IDLE);
 	}
 
 	return 0;
@@ -512,13 +518,15 @@ static struct connman_network *find_network(struct connman_device *device,
 		if (tmp_ssid && memcmp(ssid, tmp_ssid, tmp_ssid_size))
 			continue;
 
-		if (g_strcmp0(security, tmp_security) != 0)
+		if (security && tmp_security &&
+				g_strcmp0(security, tmp_security) != 0)
 			continue;
 
-		if (g_strcmp0(mode, tmp_mode) != 0)
+		if (mode && tmp_mode && g_strcmp0(mode, tmp_mode) != 0)
 			continue;
 
-		if (g_strcmp0(address, tmp_address) != 0)
+		if (address && tmp_address &&
+				g_strcmp0(address, tmp_address) != 0)
 			continue;
 
 		return connman_network_ref(value);
@@ -628,13 +636,14 @@ static DBusMessage *join_network(DBusConnection *conn,
 
 	group = build_group(ssid, ssid_size, mode, security);
 
-	connman_network_set_group(network, group);
-
 	index = connman_device_get_index(device);
 	connman_network_set_index(network, index);
 
 	connman_network_set_protocol(network, CONNMAN_NETWORK_PROTOCOL_IP);
 
+	if (connman_device_get_disconnected(device) == FALSE)
+		__connman_device_disconnect(device);
+
 	err = device->driver->join(device, network);
 
 	connman_network_unref(network);
@@ -1410,68 +1419,6 @@ int __connman_device_disconnect(struct connman_device *device)
 	return 0;
 }
 
-static void connect_known_network(struct connman_device *device)
-{
-	struct connman_network *network = NULL;
-	GHashTableIter iter;
-	gpointer key, value;
-	const char *name;
-	unsigned int count = 0;
-
-	DBG("device %p", device);
-
-	g_hash_table_iter_init(&iter, device->networks);
-
-	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
-		connman_uint8_t old_strength, new_strength;
-
-		count++;
-
-		if (connman_network_get_available(value) == FALSE)
-			continue;
-
-		name = connman_network_get_string(value,
-						CONNMAN_PROPERTY_ID_NAME);
-		if (name != NULL && device->last_network != NULL) {
-			if (g_str_equal(name, device->last_network) == TRUE) {
-				network = value;
-				break;
-			}
-		}
-
-		if (network == NULL) {
-			network = value;
-			continue;
-		}
-
-		old_strength = connman_network_get_uint8(network,
-						CONNMAN_PROPERTY_ID_STRENGTH);
-		new_strength = connman_network_get_uint8(value,
-						CONNMAN_PROPERTY_ID_STRENGTH);
-
-		if (new_strength > old_strength)
-			network = value;
-	}
-
-	if (network != NULL) {
-		int err;
-
-		name = connman_network_get_string(value,
-						CONNMAN_PROPERTY_ID_NAME);
-		if (name != NULL) {
-			err = __connman_network_connect(network);
-			if (err == 0 || err == -EINPROGRESS)
-				return;
-		}
-	}
-
-	if (count > 0)
-		return;
-
-	if (device->driver && device->driver->scan)
-		device->driver->scan(device);
-}
-
 static void mark_network_unavailable(gpointer key, gpointer value,
 							gpointer user_data)
 {
@@ -1563,8 +1510,6 @@ int connman_device_set_scanning(struct connman_device *device,
 	if (device->disconnected == TRUE)
 		return 0;
 
-	connect_known_network(device);
-
 	return 0;
 }
 
