@@ -629,7 +629,13 @@ void connman_network_set_group(struct connman_network *network,
 		__connman_profile_add_network(network);
 }
 
-const char *__connman_network_get_group(struct connman_network *network)
+/**
+ * connman_network_get_group:
+ * @network: network structure
+ *
+ * Get group name for automatic clustering
+ */
+const char *connman_network_get_group(struct connman_network *network)
 {
 	return network->group;
 }
@@ -742,14 +748,14 @@ static gboolean set_connected(gpointer user_data)
 					CONNMAN_SERVICE_STATE_CONFIGURATION);
 		}
 	} else {
-		__connman_service_indicate_state(service,
-						CONNMAN_SERVICE_STATE_IDLE);
-
 		connman_element_unregister_children(&network->element);
 
 		__connman_device_set_network(network->device, NULL);
 
 		__connman_device_decrease_connections(network->device);
+
+		__connman_service_indicate_state(service,
+						CONNMAN_SERVICE_STATE_IDLE);
 	}
 
 	return FALSE;
