@@ -23,6 +23,8 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
+#include <string.h>
 #include <gdbus.h>
 
 #include "connman.h"
@@ -44,6 +46,7 @@ struct connman_service {
 	connman_uint8_t strength;
 	connman_bool_t favorite;
 	connman_bool_t hidden;
+	connman_bool_t ignore;
 	GTimeVal modified;
 	unsigned int order;
 	char *name;
@@ -453,6 +456,17 @@ static struct connman_service *find_pending_service(void)
 	return NULL;
 }
 
+static connman_bool_t is_ignore(struct connman_service *service)
+{
+	if (service->ignore == TRUE)
+		return TRUE;
+
+	if (service->state == CONNMAN_SERVICE_STATE_FAILURE)
+		return TRUE;
+
+	return FALSE;
+}
+
 static void __connman_service_auto_connect(void)
 {
 	struct connman_service *service;
@@ -470,7 +484,7 @@ static void __connman_service_auto_connect(void)
 
 	service = g_sequence_get(iter);
 
-	while (service->state == CONNMAN_SERVICE_STATE_FAILURE) {
+	while (is_ignore(service) == TRUE) {
 		iter = g_sequence_iter_next(iter);
 		if (g_sequence_iter_is_end(iter))
 			return;
@@ -555,6 +569,8 @@ static DBusMessage *connect_service(DBusConnection *conn,
 	if (service->pending != NULL)
 		return __connman_error_in_progress(msg);
 
+	service->ignore = FALSE;
+
 	service->pending = dbus_message_ref(msg);
 
 	err = __connman_service_connect(service);
@@ -596,6 +612,8 @@ static DBusMessage *disconnect_service(DBusConnection *conn,
 		return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 	}
 
+	service->ignore = TRUE;
+
 	err = __connman_service_disconnect(service);
 	if (err < 0) {
 		if (err != -EINPROGRESS)
@@ -788,6 +806,8 @@ static void __connman_service_initialize(struct connman_service *service)
 	service->favorite = FALSE;
 	service->hidden = FALSE;
 
+	service->ignore = FALSE;
+
 	service->order = 0;
 }
 
@@ -1046,9 +1066,11 @@ int __connman_service_connect(struct connman_service *service)
 		return -EISCONN;
 
 	if (service->network != NULL) {
-		if (service->hidden == TRUE)
-			return -EINVAL;
+		unsigned int ssid_len;
 
+		if (connman_network_get_blob(service->network, "WiFi.SSID",
+					     &ssid_len) == NULL)
+			return -EINVAL;
 		connman_network_set_string(service->network,
 				"WiFi.Passphrase", service->passphrase);
 
@@ -1295,7 +1317,7 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 	if (ident == NULL)
 		return NULL;
 
-	group = __connman_network_get_group(network);
+	group = connman_network_get_group(network);
 	if (group == NULL)
 		return NULL;
 
@@ -1441,7 +1463,7 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	if (ident == NULL)
 		return NULL;
 
-	group = __connman_network_get_group(network);
+	group = connman_network_get_group(network);
 	if (group == NULL)
 		return NULL;
 
@@ -1485,6 +1507,8 @@ static int service_load(struct connman_service *service)
 	gchar *pathname, *data = NULL;
 	gsize length;
 	gchar *str;
+	unsigned int ssid_len;
+	int err = 0;
 
 	DBG("service %p", service);
 
@@ -1517,6 +1541,55 @@ static int service_load(struct connman_service *service)
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 		break;
 	case CONNMAN_SERVICE_TYPE_WIFI:
+		if (service->name == NULL) {
+			gchar *name;
+
+			name = g_key_file_get_string(keyfile,
+					service->identifier, "Name", NULL);
+			if (name != NULL) {
+				g_free(service->name);
+				service->name = name;
+			}
+
+			if (service->network != NULL)
+				connman_network_set_name(service->network,
+									name);
+		}
+
+		if (service->network &&
+				connman_network_get_blob(service->network,
+					"WiFi.SSID", &ssid_len) == NULL) {
+			gchar *hex_ssid;
+
+			hex_ssid = g_key_file_get_string(keyfile,
+							service->identifier,
+							"WiFi.SSID", NULL);
+
+			if (hex_ssid != NULL) {
+				gchar *ssid;
+				unsigned int i, j = 0, hex;
+				size_t hex_ssid_len = strlen(hex_ssid);
+
+				ssid = g_try_malloc0(hex_ssid_len / 2);
+				if (ssid == NULL) {
+					g_free(hex_ssid);
+					err = -ENOMEM;
+					goto done;
+				}
+
+				for (i = 0; i < hex_ssid_len; i += 2) {
+					sscanf(hex_ssid + i, "%02x", &hex);
+					ssid[j++] = hex;
+				}
+
+				connman_network_set_blob(service->network,
+					"WiFi.SSID", ssid, hex_ssid_len / 2);
+			}
+
+			g_free(hex_ssid);
+		}
+		/* fall through */
+
 	case CONNMAN_SERVICE_TYPE_WIMAX:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
@@ -1549,9 +1622,10 @@ static int service_load(struct connman_service *service)
 	__connman_ipconfig_load(service->ipconfig, keyfile,
 					service->identifier, "IPv4.");
 
+done:
 	g_key_file_free(keyfile);
 
-	return 0;
+	return err;
 }
 
 static int service_save(struct connman_service *service)
@@ -1560,6 +1634,7 @@ static int service_save(struct connman_service *service)
 	gchar *pathname, *data = NULL;
 	gsize length;
 	gchar *str;
+	int err = 0;
 
 	DBG("service %p", service);
 
@@ -1593,6 +1668,36 @@ update:
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 		break;
 	case CONNMAN_SERVICE_TYPE_WIFI:
+		if (service->network) {
+			const unsigned char *ssid;
+			unsigned int ssid_len = 0;
+
+			ssid = connman_network_get_blob(service->network,
+							"WiFi.SSID", &ssid_len);
+
+			if (ssid != NULL && ssid_len > 0 && ssid[0] != '\0') {
+				char *identifier = service->identifier;
+				GString *str;
+				unsigned int i;
+
+				str = g_string_sized_new(ssid_len * 2);
+				if (str == NULL) {
+					err = -ENOMEM;
+					goto done;
+				}
+
+				for (i = 0; i < ssid_len; i++)
+					g_string_append_printf(str,
+							"%02x", ssid[i]);
+
+				g_key_file_set_string(keyfile, identifier,
+							"WiFi.SSID", str->str);
+
+				g_string_free(str, TRUE);
+			}
+		}
+		/* fall through */
+
 	case CONNMAN_SERVICE_TYPE_WIMAX:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
@@ -1641,7 +1746,7 @@ done:
 
 	g_free(pathname);
 
-	return 0;
+	return err;
 }
 
 static struct connman_storage service_storage = {
